@@ -8,11 +8,11 @@
 #' @param intensity_str structure of the intensity profile, options from
 #' ('SST_array','S_ST_mat','T_SS_mat'). See 'Details'.
 #' @param pxsz size of one pixel in unit of micron, 1 for simulated data
+#' @param sz frame size of the intensity profile in x and y directions,
+#' number of pixels contained in each frame equals sz_x by sz_y.
 #' @param mindt minimum lag time, 1 for simulated data
 #' @param AIUQ_thr threshold for wave number selection, numeric vector of two
 #' elements with values between 0 and 1. See 'Details'.
-#' @param anisotropic a logical evaluating to \code{TRUE} or \code{FALSE} indicating whether
-#' anisotropic estimation should be performed
 #' @param model_name fitted model, options from  ('BM','OU','FBM','OU+FBM',
 #' 'user_defined'), with Brownian motion as the default model. See 'Details'.
 #' @param sigma_0_2_ini initial value for background noise. If NA, use minimum
@@ -35,7 +35,18 @@
 #' @param method methods for parameter estimation, options from ('AIUQ', 'DDM').
 #' @param index_q_AIUQ index range for wave number when using AIUQ method. See 'Details'.
 #' @param index_q_DDM index range for wave number when using DDM method. See 'Details'.
-#' @param message_out whether or not to output the message. 
+#' @param message_out a logical evaluating to TRUE or FALSE indicating whether
+#' or not to output the message.
+#' @param square a logical evaluating to TRUE or FALSE indicating whether or not
+#' to crop the original intensity profile into square image.
+#' @param output_dqt a logical evaluating to TRUE or FALSE indicating whether or
+#' not to compute observed dynamic image structure function(Dqt).
+#' @param output_isf a logical evaluating to TRUE or FALSE indicating whether or
+#' not to compute empirical intermediate scattering function(ISF).
+#' @param output_modeled_isf a logical evaluating to TRUE or FALSE indicating
+#' whether or not to compute modeled intermediate scattering function(ISF).
+#' @param output_modeled_dqt a logical evaluating to TRUE or FALSE indicating
+#' whether or not to compute modeled dynamic image structure function(Dqt).
 #'
 #' @details
 #' For simulated data using \code{simulation} in AIUQ package, \code{intensity}
@@ -90,11 +101,13 @@
 #' show(sim_bm)
 #' sam = SAM(sim_object = sim_bm)
 #' show(sam)
-SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(1,1),
-                anisotropic=FALSE,model_name='BM',sigma_0_2_ini=NaN,
-                param_initial=NA,num_optim=1,msd_fn=NA,msd_grad_fn=NA,num_param=NA,
+SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,sz=c(NA,NA),mindt=1,
+                AIUQ_thr=c(1,1),model_name='BM',sigma_0_2_ini=NaN,param_initial=NA,
+                num_optim=1,msd_fn=NA,msd_grad_fn=NA,num_param=NA,
                 uncertainty=FALSE,M=50,sim_object=NA, msd_truth=NA,
-                method="AIUQ",index_q_AIUQ=NA, index_q_DDM=NA,message_out=TRUE){
+                method="AIUQ",index_q_AIUQ=NA, index_q_DDM=NA,message_out=TRUE,
+                square=FALSE, output_dqt=FALSE, output_isf=FALSE,
+                output_modeled_isf=FALSE,output_modeled_dqt=FALSE){
 
   model <- methods::new("SAM")
   #check
@@ -127,30 +140,30 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
     stop("AIUQ threshold has value between 0 and 1. \n")
   }
 
-  if(!is.logical(anisotropic)){
-    stop("Indicator for anisotropic processes should be a logical value. \n")
-  }
   if(!is.numeric(sigma_0_2_ini)){
     stop("Inital value for background noise should be numeric. \n")
   }
 
-  if(isS4(sim_object)){
+  if(class(sim_object)[1]=="simulation"){
     intensity = sim_object@intensity
     model@pxsz = sim_object@pxsz
-    model@mindt = sim_object@mindt
+    #model@mindt = sim_object@mindt
     M = sim_object@M
     #model_name = sim_object@model_name ##not always inherent
     len_t = sim_object@len_t
     model@param_truth = get_true_param_sim(param_truth=sim_object@param,model_name=sim_object@model_name)
     model@msd_truth = sim_object@theor_msd
     model@sigma_2_0_truth = sim_object@sigma_2_0
+    sz = sim_object@sz
   }else{
     model@pxsz = pxsz
-    model@mindt = mindt
+    #model@mindt = mindt
     model@msd_truth = msd_truth
     model@sigma_2_0_truth = NA
     model@param_truth = NA
+    sz = sz
   }
+  model@mindt = mindt
   if(is.vector(intensity)){
     if(is.na(intensity)){
       stop("Intensity profile can't be missing and should have one of the structure listed in intensity_str. \n")
@@ -171,17 +184,21 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
     stop("Method should be character. \n")
   }
 
-  model@anisotropic = anisotropic
   model@model_name = model_name
   model@method = method
 
   # Transform intensity into the same format and crop image into square image
-  # total number of pixels in each image = sz^2
-  intensity_transform = intensity_format_transform(intensity = intensity,intensity_str = intensity_str)
+  # total number of pixels in each image = sz_x*sz_y
+  intensity_list = intensity_format_transform(intensity = intensity,
+                                              intensity_str = intensity_str,
+                                              square = square,sz=sz)
 
   # Fourier transform
-  fft_list = FFT2D(intensity=intensity_transform,pxsz=model@pxsz,mindt=model@mindt)
-  model@sz = fft_list$sz
+  fft_list = FFT2D(intensity_list=intensity_list,pxsz=model@pxsz,mindt=model@mindt)
+
+  #num of rows and columns of intensity matrix, also representing frame size in y and x directions
+  model@sz = c(fft_list$sz_y,fft_list$sz_x)
+
   model@len_q = fft_list$len_q
   model@len_t = fft_list$len_t
   model@q = fft_list$q
@@ -202,13 +219,21 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
 
 
   # get each q ring location index
-  v = (-(model@sz-1)/2):((model@sz-1)/2)
-  x = matrix(rep(v,each = model@sz), byrow = FALSE,nrow = model@sz)
-  y = matrix(rep(v,each = model@sz), byrow = TRUE,nrow = model@sz)
+  if(model@sz[1]==model@sz[2]){
+    v = (-(model@sz[1]-1)/2):((model@sz[1]-1)/2)
+    x = matrix(rep(v,each = model@sz[1]), byrow = FALSE,nrow = model@sz[1])
+    y = matrix(rep(v,each = model@sz[1]), byrow = TRUE,nrow = model@sz[1])
+  }else{
+    v_x = (-(model@sz[2]-1)/2):((model@sz[2]-1)/2)
+    v_y = (-(model@sz[1]-1)/2):((model@sz[1]-1)/2)
+    x = matrix(rep(v_x,each = model@sz[1]), byrow = FALSE,nrow = model@sz[1])
+    y = matrix(rep(v_y,each = model@sz[2]), byrow = TRUE,nrow = model@sz[1])
+  }
   theta_q = cart2polar(x, y)
-  q_ring_num = theta_q[,(model@sz+1):dim(theta_q)[2]]
+  q_ring_num = theta_q[,(model@sz[2]+1):dim(theta_q)[2]]
   q_ring_num = round(q_ring_num)
 
+
   nq_index = vector(mode = "list")
   for(i in 1:model@len_q){
     nq_index[[i]] = which(q_ring_num==i)
@@ -232,7 +257,7 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
   avg_I_2_ori = 0
 
   for(i in 1:model@len_t){
-    avg_I_2_ori = avg_I_2_ori+abs(fft_list$I_q_matrix[,i])^2/model@sz^2
+    avg_I_2_ori = avg_I_2_ori+abs(fft_list$I_q_matrix[,i])^2/(model@sz[1]*model@sz[2])
   }
   avg_I_2_ori = avg_I_2_ori/model@len_t
 
@@ -284,7 +309,7 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
                                       sigma_0_2_ini=sigma_0_2_ini,
                                       num_param=num_param)
   }else{
-    param_initial = c(param_initial,sigma_0_2_ini)
+    param_initial = log(c(param_initial,sigma_0_2_ini))
   }
 
   if(model@method == "AIUQ"){
@@ -439,21 +464,52 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
       model@msd_upper = NA
       model@param_uq_range = matrix(NA,1,1)
     }
-    Dqt = matrix(NA,1,1)
+
+    if(output_dqt==FALSE && output_isf==FALSE){
+      Dqt = matrix(NA,1,1)
+      isf = matrix(NA,1,1)
+    }else if(output_dqt==TRUE && output_isf==FALSE){
+      Dqt = SAM_Dqt(len_q=model@len_q,index_q=1:model@len_q,len_t=model@len_t,
+                    I_q_matrix=fft_list$I_q_matrix,sz=model@sz,
+                    q_ori_ring_loc_unique_index=q_ori_ring_loc_unique_index)
+      isf = matrix(NA,1,1)
+    }else if(output_isf==TRUE){
+      Dqt = matrix(NA,model@len_q,model@len_t-1)
+      isf = matrix(NA,model@len_q,model@len_t-1)
+      for (q_j in 1:model@len_q){
+        index_cur = q_ori_ring_loc_unique_index[[q_j]]
+        I_q_cur = fft_list$I_q_matrix[index_cur,]
+        for (t_i in 1:(model@len_t-1)){
+          Dqt[q_j,t_i]=mean((abs(I_q_cur[,(t_i+1):model@len_t]-I_q_cur[,1:(model@len_t-t_i)]))^2/(model@sz[1]*model@sz[2]),na.rm=T)
+        }
+        if(model@A_est_ini[q_j]==0){break}
+        isf[q_j,] = 1-(Dqt[q_j,]-model@B_est_ini)/model@A_est_ini[q_j]
+      }
+    }
+
   }else if(model@method == "DDM_fixedAB"){
     if(length(index_q_DDM)==1 &&is.na(index_q_DDM)){
       index_q_DDM = 1:model@len_q
     }
-    Dqt = get_Dqt(len_q=model@len_q,index_q=index_q_DDM,len_t=model@len_t,
+    Dqt = SAM_Dqt(len_q=model@len_q,index_q=index_q_DDM,len_t=model@len_t,
                   I_q_matrix=fft_list$I_q_matrix,sz=model@sz,
                   q_ori_ring_loc_unique_index=q_ori_ring_loc_unique_index)
+    if(output_isf==TRUE){
+      isf = matrix(NA,model@len_q,model@len_t-1)
+      for (q_j in 1:model@len_q){
+        if(model@A_est_ini[q_j]==0){break}
+        isf[q_j,] = 1-(Dqt[q_j,]-model@B_est_ini)/model@A_est_ini[q_j]
+        }
+      }else{
+        isf = matrix(NA,1,1)
+      }
+
     l2_est_list = theta_est_l2_dqt_fixedAB(param=param_initial[-length(param_initial)],q=model@q,index_q=index_q_DDM,
                                            Dqt=Dqt,A_est_q=model@A_est_ini,B_est=model@B_est_ini,
                                            d_input=model@d_input, model_name=model@model_name,
                                            msd_fn=msd_fn,msd_grad_fn=msd_grad_fn)
 
 
-
     model@param_est = l2_est_list$param_est
     model@msd_est = l2_est_list$msd_est
     model@sigma_2_0_est = model@B_est_ini/2
@@ -465,9 +521,19 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
     if(length(index_q_DDM)==1 &&is.na(index_q_DDM)){
       index_q_DDM = 1:model@len_q
     }
-    Dqt = get_Dqt(len_q=model@len_q,index_q=index_q_DDM,len_t=model@len_t,
+    Dqt = SAM_Dqt(len_q=model@len_q,index_q=index_q_DDM,len_t=model@len_t,
                   I_q_matrix=fft_list$I_q_matrix,sz=model@sz,
                   q_ori_ring_loc_unique_index=q_ori_ring_loc_unique_index)
+    if(output_isf==TRUE){
+      isf = matrix(NA,model@len_q,model@len_t-1)
+      for (q_j in 1:model@len_q){
+        if(model@A_est_ini[q_j]==0){break}
+        isf[q_j,] = 1-(Dqt[q_j,]-model@B_est_ini)/model@A_est_ini[q_j]
+      }
+    }else{
+      isf = matrix(NA,1,1)
+    }
+
     l2_est_list = theta_est_l2_dqt_estAB(param=param_initial,q=model@q,index_q=index_q_DDM,
                                          Dqt=Dqt,A_ini=model@A_est_ini,d_input=model@d_input,
                                          model_name=model@model_name,msd_fn=msd_fn,msd_grad_fn=msd_grad_fn)
@@ -481,7 +547,32 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
     model@mle = NaN
     model@param_uq_range = matrix(NA,1,1)
   }
+
   model@Dqt = Dqt
+  model@ISF = isf
+
+  if(output_modeled_dqt==FALSE && output_modeled_isf==FALSE){
+    model@modeled_Dqt = matrix(NA,1,1)
+    model@modeled_ISF = matrix(NA,1,1)
+  }else if(output_modeled_isf==TRUE && output_modeled_dqt==FALSE){
+    model@modeled_ISF = matrix(NA,model@len_q,model@len_t-1)
+    model@modeled_Dqt = matrix(NA,1,1)
+    for(q_j in 1:model@len_q){
+      q_selected = model@q[q_j]
+      model@modeled_ISF [q_j,] = exp(-q_selected^2*model@msd_est[-1]/4)
+    }
+  }else if(output_modeled_dqt==TRUE){
+    model@modeled_ISF  = matrix(NA,model@len_q,model@len_t-1)
+    model@modeled_Dqt = matrix(NA,model@len_q,model@len_t-1)
+    for(q_j in 1:model@len_q){
+      q_selected = model@q[q_j]
+      model@modeled_ISF[q_j,] = exp(-q_selected^2*model@msd_est[-1]/4)
+      if(model@A_est_ini[q_j]==0){break}
+      model@modeled_Dqt[q_j,] = model@A_est_ini[q_j]*(1-model@modeled_ISF[q_j,])+model@sigma_2_0_est*2
+    }
+
+  }
+
   if(model@method=='AIUQ'){
      model@index_q = index_q_AIUQ
   }else{ ##DDM
@@ -491,5 +582,6 @@ SAM <- function(intensity=NA,intensity_str="T_SS_mat",pxsz=1,mindt=1,AIUQ_thr=c(
   model@I_q = fft_list$I_q_matrix
   #model@p = p
   model@AIC = AIC
+  model@q_ori_ring_loc_unique_index = q_ori_ring_loc_unique_index
   return(model)
 }
